@@ -1,4 +1,4 @@
-/* psync.c - Last modified: 14-Apr-2019 (kobayasy)
+/* psync.c - Last modified: 13-Jun-2019 (kobayasy)
  *
  * Copyright (c) 2018-2019 by Yuichi Kobayashi <kobayasy@kobayasy.com>
  *
@@ -359,6 +359,7 @@ static int delete_func(FLIST *f, void *data) {
 #define UPFILE   "u%lu"
 #define DOWNFILE "d%lu"
 #define BACKFILE "%lu,%s"
+#define LOGFILE  "log"
 typedef struct {
     time_t t;
     time_t expire;
@@ -394,15 +395,16 @@ static int progress_init(PROGRESS *progress, int fd, unsigned long interval, con
     int status = -1;
     struct timespec now;
 
-    if (fd != -1) {
-        if (clock_gettime(CLOCK_REALTIME, &now) == -1)
-            goto error;
-        progress->fd = fd;
+    progress->fd = fd;
+    if (progress->fd != -1) {
         progress->interval = interval;
         progress->format = format;
         progress->update = update;
-        progress->data = progress->update;
-        progress->last = now;
+        if (clock_gettime(CLOCK_REALTIME, &now) == -1) {
+            progress->fd = -1;
+            goto error;
+        }
+        progress->data = progress->update, progress->last = now;
     }
     status = 0;
 error:
@@ -420,8 +422,7 @@ static int progress_update(PROGRESS *progress, intmax_t update) {
                 goto error;
             if (DIFFTS(now, progress->last) >= progress->interval) {
                 dprintf(progress->fd, progress->format, progress->update);
-                progress->data = progress->update;
-                progress->last = now;
+                progress->data = progress->update, progress->last = now;
             }
         }
     }
@@ -1131,6 +1132,143 @@ error:
     return status;
 }
 
+static int make_log(PRIV *priv) {
+    int status = INT_MIN;
+    char pathname[PATH_MAX];
+    FILE *fp = NULL;
+    struct {
+        unsigned long backup;
+        unsigned long deleted;
+        unsigned long added;
+        unsigned long modified;
+        unsigned long uploaded;
+    } count;
+    FLIST *fsynced;
+
+    sprintf(pathname, "%s/"SYNCDIR"/"LOCKDIR"/"LOGFILE, priv->dirname);
+    fp = fopen(pathname, "w");
+    if (fp == NULL) {
+        status = ERROR_DMAKE;
+        goto error;
+    }
+    memset(&count, 0, sizeof(count));
+    for (fsynced = priv->fsynced.next; *fsynced->name; fsynced = fsynced->next) {
+        ONSTOP(priv->stop, ERROR_STOP);
+        switch (fsynced->st.flags & (FST_DNLD|FST_RTYPE|FST_LTYPE)) {
+        case FST_DNLD|FST_LREG:
+            ++count.backup;
+        case FST_DNLD|FST_LDIR:
+        case FST_DNLD|FST_LLNK:
+            ++count.deleted;
+            break;
+        case FST_DNLD|FST_RREG:
+        case FST_DNLD|FST_RDIR:
+        case FST_DNLD|FST_RLNK:
+            ++count.added;
+            break;
+        case FST_DNLD|FST_LREG|FST_RREG:
+        case FST_DNLD|FST_LREG|FST_RDIR:
+        case FST_DNLD|FST_LREG|FST_RLNK:
+            ++count.backup;
+        case FST_DNLD|FST_LDIR|FST_RREG:
+        case FST_DNLD|FST_LDIR|FST_RDIR:
+        case FST_DNLD|FST_LDIR|FST_RLNK:
+        case FST_DNLD|FST_LLNK|FST_RREG:
+        case FST_DNLD|FST_LLNK|FST_RDIR:
+        case FST_DNLD|FST_LLNK|FST_RLNK:
+            ++count.modified;
+            break;
+        }
+        switch (fsynced->st.flags & FST_UPLD) {
+        case FST_UPLD:
+            ++count.uploaded;
+            break;
+        }
+    }
+    fprintf(fp, "%lu deleted, %lu added, %lu modified, %lu uploaded\n",
+            count.deleted, count.added, count.modified, count.uploaded );
+    for (fsynced = priv->fsynced.next; *fsynced->name; fsynced = fsynced->next) {
+        ONSTOP(priv->stop, ERROR_STOP);
+        switch (fsynced->st.flags & (FST_DNLD|FST_RTYPE|FST_LTYPE)) {
+        case FST_DNLD|FST_LREG:
+            strcpy(pathname, fsynced->name);
+            fprintf(fp, "D %s -> %lu,%s\n", fsynced->name, count.backup--, basename(pathname));
+            break;
+        case FST_DNLD|FST_LDIR:
+            fprintf(fp, "D %s/\n", fsynced->name);
+            break;
+        case FST_DNLD|FST_LLNK:
+            fprintf(fp, "D %s@\n", fsynced->name);
+            break;
+        }
+    }
+    for (fsynced = priv->fsynced.next; *fsynced->name; fsynced = fsynced->next) {
+        ONSTOP(priv->stop, ERROR_STOP);
+        switch (fsynced->st.flags & (FST_DNLD|FST_RTYPE|FST_LTYPE)) {
+        case FST_DNLD|FST_RREG:
+            fprintf(fp, "A %s\n", fsynced->name);
+            break;
+        case FST_DNLD|FST_RDIR:
+            fprintf(fp, "A %s/\n", fsynced->name);
+            break;
+        case FST_DNLD|FST_RLNK:
+            fprintf(fp, "A %s@\n", fsynced->name);
+            break;
+        }
+    }
+    for (fsynced = priv->fsynced.next; *fsynced->name; fsynced = fsynced->next) {
+        ONSTOP(priv->stop, ERROR_STOP);
+        switch (fsynced->st.flags & (FST_DNLD|FST_RTYPE|FST_LTYPE)) {
+        case FST_DNLD|FST_RREG|FST_LREG:
+            strcpy(pathname, fsynced->name);
+            fprintf(fp, "M %s -> %lu,%s\n", fsynced->name, count.backup--, basename(pathname));
+            break;
+        case FST_DNLD|FST_RREG|FST_LDIR:
+        case FST_DNLD|FST_RREG|FST_LLNK:
+            fprintf(fp, "M %s\n", fsynced->name);
+            break;
+        case FST_DNLD|FST_RDIR|FST_LREG:
+            strcpy(pathname, fsynced->name);
+            fprintf(fp, "M %s/ -> %lu,%s\n", fsynced->name, count.backup--, basename(pathname));
+            break;
+        case FST_DNLD|FST_RDIR|FST_LDIR:
+        case FST_DNLD|FST_RDIR|FST_LLNK:
+            fprintf(fp, "M %s/\n", fsynced->name);
+            break;
+        case FST_DNLD|FST_RLNK|FST_LREG:
+            strcpy(pathname, fsynced->name);
+            fprintf(fp, "M %s@ -> %lu,%s\n", fsynced->name, count.backup--, basename(pathname));
+            break;
+        case FST_DNLD|FST_RLNK|FST_LDIR:
+        case FST_DNLD|FST_RLNK|FST_LLNK:
+            fprintf(fp, "M %s@\n", fsynced->name);
+            break;
+        }
+    }
+    for (fsynced = priv->fsynced.next; *fsynced->name; fsynced = fsynced->next) {
+        ONSTOP(priv->stop, ERROR_STOP);
+        switch (fsynced->st.flags & (FST_UPLD|FST_LTYPE)) {
+        case FST_UPLD|FST_LREG:
+            fprintf(fp, "U %s\n", fsynced->name);
+            break;
+        case FST_UPLD|FST_LDIR:
+            fprintf(fp, "U %s/\n", fsynced->name);
+            break;
+        case FST_UPLD|FST_LLNK:
+            fprintf(fp, "U %s@\n", fsynced->name);
+            break;
+        case FST_UPLD:
+            fprintf(fp, "U %s%%\n", fsynced->name);
+            break;
+        }
+    }
+    status = 0;
+error:
+    if (fp != NULL)
+        fclose(fp);
+    return status;
+}
+
 static int clean_r(char *pathname, char *name,
                    volatile sig_atomic_t *stop ) {
     int status = INT_MIN;
@@ -1314,6 +1452,10 @@ static int run(int (*func)(FSETS sets, FLIST *f1, FLIST *f2, void *data), PRIV *
     if (ISERR(status))
         goto error;
     ONSTOP(priv->stop, ERROR_STOP);
+    status = make_log(priv);
+    if (ISERR(status))
+        goto error;
+    ONSTOP(priv->stop, ERROR_STOP);
     status = clean(priv);
     if (ISERR(status))
         goto error;
